
/**
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: httpd-fs.c 10484 2010-10-22 07:39:23Z shnaz $
*/

//#pragma GCC optimize ("0")


#define HTTPD_DEBUG 		            LWIP_DBG_OFF
#define  NO_SDCARD

#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>


#include "httpd.h"
#include "httpd-fs.h"
#include "fsdata404.h"

#include "lwip/opt.h"
#include "lwip/arch.h"
#include "lwip/api.h"
#include "lwip/err.h"


#include "httpd-fs.h"
#include "httpd.h"
#include "lwip/mem.h"
#include "lwipopts.h"

#include "esp_vfs_fat.h"
#include "cgiScripts.h"
#include "BBQ1data.h" ; // actually c code.. containing build-in website

static bool sendCGIfile;

#define mem_malloc malloc
#define mem_free free


void fstime(void) {
}

//*****************************************************************************
//
// The following are data structures used by FatFs.
//
//*****************************************************************************


/**
 * Open a file and return a handle to the file, if found. Otherwise, return NULL.
 * if filename == CGIRETURNFILE then sendCGIfile is set, file read will then execute a script instead of reading a file
 * @param[in] name char pointer to filename
 * param[out] handle to the file, if found. Otherwise NULL.
 */
struct fs_file * fs_open(char *name) {
	const struct fsdata_file *ptTree;
	struct fs_file *ptFile = NULL;
	FIL *ptFatFile = NULL;
	FRESULT fresult = FR_OK;

	//
	// Allocate memory for the file system structure.
	//
	ptFile = mem_malloc(sizeof(struct fs_file));
	if (NULL == ptFile) {
		//	while(1);
		return (NULL);
	}

	ptFatFile = mem_malloc(sizeof(FIL));
	if (NULL == ptFatFile) {
		mem_free(ptFile);
		ptFile = NULL;
		//	while(1);
		return (NULL);
	}

	//
	// Check to see if the Fat File System has been enabled.
	//
	if (strcmp(name, CGIRETURNFILE) == 0) { // this filename is used to generate various cgi response data
		sendCGIfile = true;
		ptFile->data = NULL;
		ptFile->len = 0;
		ptFile->index = 0;
		ptFile->pextension = ptFatFile;
		return (ptFile);
	}
	//
	// Attempt to open the file on the Fat File System.
	//

#ifndef NO_SDCARD
	fresult = f_open(ptFatFile, name, FA_READ | FA_OPEN_EXISTING);
	if (FR_OK == fresult) {
		ptFile->data = NULL;
		ptFile->len = 0;
		ptFile->index = 0;
		ptFile->pextension = ptFatFile;
		return (ptFile);
	}
	//
	// If we get here, we failed to find the file on the Fat File System,
	// so free up the Fat File system handle/object.
	//
#endif

	mem_free(ptFatFile);
	//
	// Initialize the file system tree pointer to the root of the linked list.
	//
	ptTree = FS_ROOT;
	//
	// Begin processing the linked list, looking for the requested file name.
	//
	while (NULL != ptTree) {
		//
		// Compare the requested file "name" to the file name in the
		// current node.
		//
		if (strncmp(name, (char *) ptTree->name, ptTree->len) == 0)
		{
			//
			// Fill in the data pointer and length values from the
			// linked list node.
			//
			LWIP_DEBUGF(HTTPD_DEBUG, ("fs openend %s \n", name));
			ptFile->data = (char *) ptTree->data;
			ptFile->len = ptTree->len;
			ptFile->index = 0;
			ptFile->pextension = NULL;
			break;
		}
		//
		// If we get here, we did not find the file at this node of the linked
		// list.  Get the next element in the list.
		//
		ptTree = ptTree->next;
	}
	//
	// If we didn't find the file, ptTee will be NULL.  Make sure we
	// return a NULL pointer if this happens.
	//
	if (NULL == ptTree) {
		mem_free(ptFile);
		ptFile = NULL;
	}
	//
	// Return the file system pointer.
	//
	return (ptFile);
}


/**
 * close a file
 * @param[in] file handle to the file to close
 */

void fs_close(struct fs_file *file) {
	//
	// If a Fat file was opened, free its object.
	//
	if (file->pextension) {
		mem_free(file->pextension);
		file->pextension = NULL;
	}
	mem_free(file);
}

//*****************************************************************************
//
// Read the next chunck of data from the file.  Return the count of data
// that was read.  Return 0 if no data is currently available.  Return
// a -1 if at the end of file.
//
//*****************************************************************************

#define MAXBYTESTOREAD 1024

/**
 * reads a number of bytes from file or when sendCGIfile is true from a cgi-script
 * @param[in] file handle to the file to read
 * @param[in] buffer pointer to buffer to write in
 * @param[in] count max number of bytes to read
 * @param[out] nr nr of bytes read
 */

int fs_read(struct fs_file *file, char *buffer, int count) {
	int iAvailable;
	uint32_t usBytesRead;
	//
	// Check to see if a Fat File was opened and process it.
	//
	if (file->pextension) {
		FRESULT fresult;

		if ( sendCGIfile){  // reading selfmade CGIRETURNFILE
			usBytesRead = readResponseFile(buffer, count);
			if (usBytesRead == 0){
				sendCGIfile = false;
				return EOF;
			}
			return usBytesRead;
		}

		fresult = f_read(file->pextension, buffer, count, &usBytesRead);
		if ((fresult != FR_OK) || (usBytesRead == 0)) {
			//fs_close( file); done later
			return (-1);
		}
		return (usBytesRead);
	}
	// Check to see if more data is available.
	//
	if (file->len == file->index) {
		//
		// There is no remaining data.  Return a -1 for EOF indication.
		//
		return (-1);
	}
	//
	// Determine how much data we can copy.  The minimum of the 'count'
	// parameter or the available data in the file system buffer.
	//
	iAvailable = file->len - file->index;
	if (iAvailable > count) {
		iAvailable = count;
	}
	// Copy the data.

	memcpy(buffer, file->data + file->index, iAvailable);
	file->index += iAvailable;
	//
	// Return the count of data that we copied.
	//
	return (iAvailable);
}

